@@ -14,6 +14,7 @@ from collections import OrderedDict
 from .mutables import Mutable, MutableScope
 from ..utils.common import generate_id, combinations
 from ..utils import logging
+from .random_state import get_random_state
 
 logger = logging.get_logger(__name__)
 
@@ -24,6 +25,11 @@ class HyperNode(Mutable):
         Mutable.__init__(self, self._space.scope, name)
         self._space.add_node(self)
 
+    def attach_to_space(self, space=None, name=None):
+        self._space = space if space is not None else get_default_space()
+        self.attach_to_scope(self._space.scope, name)
+        self._space.add_node(self)
+
     @property
     def space(self):
         return self._space
@@ -98,12 +104,13 @@ class HyperSpace(Mutable):
     def _compile_space(self):
         assert not self._is_compiled, 'HyperSpace does not allow to compile repeatedly.'
         space_out = []
-        counter = []
+        counter = 0
         start_ts = time.time()
 
         def compile_module(module):
             module.compile()
-            counter.append(0)
+            nonlocal counter
+            counter += 1
             if len(self.get_outputs(module)) <= 0:
                 space_out.append(module)
             return True
@@ -111,7 +118,7 @@ class HyperSpace(Mutable):
         self.traverse(compile_module, direction='forward')
         end_ts = time.time()
         if logger.is_info_enabled():
-            logger.debug(f'Compile Space: compiled {len(counter)} modules in {end_ts - start_ts} seconds.')
+            logger.debug(f'Compile Space: compiled {counter} modules in {end_ts - start_ts} seconds.')
         self._is_compiled = True
         self._outputs = set(space_out)
 
@@ -468,6 +475,36 @@ class HyperSpace(Mutable):
             count *= hp.choice_num
         return count
 
+    def _repr_html_(self):
+        html = '''<table border="1" class="dataframe">
+<thead>
+<tr style="text-align: right;">
+  <th>key</th>
+  <th>value</th>
+</tr>
+</thead>
+<tbody>'''
+        html += f'''<tr>
+  <td>signature</td>
+  <td>{self.signature}</td>
+</tr>
+<tr>
+  <td>vectors</td>
+  <td>{self.vectors}</td>
+</tr>'''
+        params = self.get_assigned_params()
+        for i, hp in enumerate(params):
+            html += f'''<tr>
+  <td>{i}-{hp.alias}</td>
+  <td>{hp.value}</td>
+</tr>
+<tr>'''
+
+        html += '''  </tbody>
+</table>
+</div>'''
+        return html
+
 
 class DefaultStack(threading.local):
     def __init__(self):
@@ -531,10 +568,11 @@ def get_default_space():
 
 
 class ParameterSpace(HyperNode):
-    def __init__(self, space=None, name=None):
+    def __init__(self, space=None, name=None, random_state=None):
         HyperNode.__init__(self, space, name)
         self._assigned = False
         self._value = None
+        self.random_state = random_state if random_state is not None else get_random_state()
         self.references = set()
 
     @property
@@ -624,14 +662,13 @@ class ParameterSpace(HyperNode):
 
 
 class Int(ParameterSpace):
-    def __init__(self, low, high, step=1, random_state=np.random.RandomState(), space=None, name=None):
-        ParameterSpace.__init__(self, space, name)
+    def __init__(self, low, high, step=1, random_state=None, space=None, name=None):
+        ParameterSpace.__init__(self, space, name, random_state)
         assert isinstance(low, int) and isinstance(high, int), '`low` and `high` must be a int.'
         assert low < high, '`low` must less than `high`.'
         self.low = low
         self.high = high
         self.step = step
-        self.random_state = random_state
 
     def _random_sample(self):
         value = self.random_state.randint(self.low, self.high)
@@ -679,9 +716,8 @@ class Int(ParameterSpace):
 
 class Real(ParameterSpace):
     def __init__(self, low, high, q=None, prior="uniform", step=0.01, max_expansion=100,
-                 random_state=np.random.RandomState(),
-                 space=None, name=None):
-        ParameterSpace.__init__(self, space, name)
+                 random_state=None, space=None, name=None):
+        ParameterSpace.__init__(self, space, name, random_state)
         low = float(low)
         high = float(high)
         assert low < high, '`low` must less than `high`.'
@@ -690,7 +726,6 @@ class Real(ParameterSpace):
         self.q = q
         self.prior = prior
         self.step = float(step)
-        self.random_state = random_state
         self.max_expansion = max_expansion
 
     def _random_sample(self):
@@ -711,12 +746,12 @@ class Real(ParameterSpace):
 
         if self.step is not None:
             if self.prior == 'log_uniform':
-                all = np.arange(np.exp(self.low), np.exp(self.high) + self.step, step=self.step)
+                all = np.round(np.arange(np.exp(self.low), np.exp(self.high) + self.step, step=self.step), 8)
                 value = all[np.abs(all - value).argmin()]
                 if value > np.exp(self.high):
                     value = np.exp(self.high)
             else:
-                all = np.arange(self.low, self.high + self.step, step=self.step)
+                all = np.round(np.arange(self.low, self.high + self.step, step=self.step), 8)
                 value = all[np.abs(all - value).argmin()]
                 if value > self.high:
                     value = self.high
@@ -765,12 +800,11 @@ class Real(ParameterSpace):
 
 
 class Choice(ParameterSpace):
-    def __init__(self, options, random_state=np.random.RandomState(), space=None, name=None):
-        ParameterSpace.__init__(self, space, name)
+    def __init__(self, options, random_state=None, space=None, name=None):
+        ParameterSpace.__init__(self, space, name, random_state)
         assert isinstance(options, list), '`options` must be a List.'
         assert len(options) > 0, '`options` contains at least one item.'
         self.options = options
-        self.random_state = random_state
 
     @property
     def is_mutable(self):
@@ -806,15 +840,14 @@ class Choice(ParameterSpace):
 
 
 class MultipleChoice(ParameterSpace):
-    def __init__(self, options, num_chosen_most=0, num_chosen_least=1, random_state=np.random.RandomState(),
+    def __init__(self, options, num_chosen_most=0, num_chosen_least=1, random_state=None,
                  space=None, name=None):
-        ParameterSpace.__init__(self, space, name)
+        ParameterSpace.__init__(self, space, name, random_state)
         assert isinstance(options, list), '`options` must be a List.'
         assert len(options) >= num_chosen_least, f'`options` contains at least {num_chosen_least} item.'
         self.options = options
         self.num_chosen_most = num_chosen_most
         self.num_chosen_least = num_chosen_least
-        self.random_state = random_state
 
     def _random_sample(self):
         high = self.num_chosen_most
@@ -869,7 +902,7 @@ class MultipleChoice(ParameterSpace):
 
 
 class Bool(Choice):
-    def __init__(self, random_state=np.random.RandomState(), space=None, name=None):
+    def __init__(self, random_state=None, space=None, name=None):
         Choice.__init__(self, [False, True], random_state, space, name)
 
 
