@@ -4,8 +4,12 @@ __author__ = 'yangjian'
 
 """
 import time
+import json
 
-from hypernets.utils import logging
+from IPython.display import display
+
+from hypernets.dispatchers.cfg import DispatchCfg
+from hypernets.utils import logging, df_utils
 
 logger = logging.get_logger(__name__)
 
@@ -33,11 +37,11 @@ class ExperimentCallback():
         pass
 
 
-class Experiment():
+class Experiment(object):
     def __init__(self, hyper_model, X_train, y_train, X_eval=None, y_eval=None, X_test=None, eval_size=0.3,
-                 task=None, callbacks=None, random_state=9527):
+                 task=None, id=None, callbacks=None, random_state=9527):
         self.task = task
-        self.id = None
+        self.id = id
         self.title = None
         self.description = None
         self.dataset_id = None
@@ -52,8 +56,8 @@ class Experiment():
         self.y_eval = y_eval
         self.X_test = X_test
 
-        if X_eval is not None and X_test is None:
-            self.X_test = X_eval
+        # if X_eval is not None and X_test is None:
+        #     self.X_test = X_eval
 
         self.eval_size = eval_size
         self.callbacks = callbacks if callbacks is not None else []
@@ -61,9 +65,17 @@ class Experiment():
 
         self.start_time = None
         self.end_time = None
+    
+    def get_data_character(self):
+        data_character = df_utils.get_data_character(self.hyper_model, self.X_train, self.y_train, self.X_eval,
+                                                     self.y_eval, self.X_test, self.task)
+        return data_character
 
     def run(self, **kwargs):
         self.start_time = time.time()
+
+        DispatchCfg.experiment = str(self.id) if self.id is not None else ''
+
         try:
             if self.task is None:
                 self.task, _ = self.hyper_model.infer_task_type(self.y_train)
@@ -88,7 +100,8 @@ class Experiment():
 
     def step_start(self, step):
         if self.current_step is not None:
-            raise RuntimeError('Make sure `step_end()` is called at the end of the previous step.')
+            raise RuntimeError(
+                f'Make sure `step_end()` is called at the end of the previous step. `step`:{step}, `current_step`:{self.current_step}')
 
         self.current_step = step
         self.step_start_time = time.time()
@@ -104,6 +117,15 @@ class Experiment():
         self.current_step = None
         self.step_start_time = None
 
+    def step_break(self, error=None):
+        if self.current_step is None:
+            raise RuntimeError('Make sure `step_start()` is called at the start of the current step.')
+
+        for callback in self.callbacks:
+            callback.step_break(self, self.current_step, error)
+        self.current_step = None
+        self.step_start_time = None
+
     def step_progress(self, progress, eta=None):
         elapsed = time.time() - self.step_start_time
         for callback in self.callbacks:
@@ -133,3 +155,9 @@ class Experiment():
             return -1
         else:
             return self.end_time - self.start_time
+
+    def plot_dataset(self):
+        data = self.get_data_character()
+        from hn_widget.widget import DatasetSummary
+        widget = DatasetSummary(data)
+        display(widget)
