@@ -5,11 +5,32 @@
 
 import json
 import os
+import sys
 import tempfile
 
 import fsspec
 
-from .common import config
+from hypernets.conf import Configurable, configure, Unicode
+from . import logging
+
+logger = logging.get_logger(__name__)
+is_windows = sys.platform.find('win') >= 0
+
+
+@configure()
+class StorageCfg(Configurable):
+    kind = Unicode('file',
+                   help='storage kind (fsspec protocol). "file" and "s3" were tested.'
+                   ).tag(config=True)
+    root = Unicode('',
+                   help='storage root path.'
+                   ).tag(config=True)
+    options = Unicode('',
+                      help='storage options, json string. see fsspec from more details.'
+                      ).tag(config=True)
+    local_root = Unicode(os.path.join(tempfile.gettempdir(), 'cache'),
+                         help='local root path, used as temp.'
+                         ).tag(config=True)
 
 
 class FileSystemAdapter(object):
@@ -20,14 +41,31 @@ class FileSystemAdapter(object):
         self.remote_root = remote_root
         self.local_root = local_root
         self.remote_sep = remote_sep
+        self.remote_root_alias = remote_root.replace('\\', '/') if is_windows else None
+
+    def _inner_to_rpath(self, rpath):
+        if rpath.startswith(self.remote_root):
+            return rpath
+
+        if self.remote_root_alias and rpath.startswith(self.remote_root_alias):
+            return rpath
+
+        if is_windows and rpath.find(':') > 0:
+            return rpath
+
+        return self.remote_root.rstrip(self.remote_sep) + self.remote_sep + rpath.lstrip(self.remote_sep)
 
     def to_rpath(self, rpath):
         # return os.path.join(remote_root, rpath)
         assert rpath
-        if rpath.startswith(self.remote_root):
-            return rpath
 
-        return self.remote_root.rstrip(self.remote_sep) + self.remote_sep + rpath.lstrip(self.remote_sep)
+        if isinstance(rpath, str):
+            return self._inner_to_rpath(rpath)
+        elif isinstance(rpath, (list, tuple)):
+            return [self._inner_to_rpath(p) for p in rpath]
+        else:
+            logger.warn(f'Unexpected rpath type: {type(rpath)}, rpath: {rpath}')
+            return rpath
 
     def to_lpath(self, lpath):
         assert lpath
@@ -259,31 +297,42 @@ def get_filesystem(fs_type, fs_root, fs_options) -> fsspec.AbstractFileSystem:
     if type(fs).__name__.lower().find('local') >= 0:
         if fs_root is None or fs_root == '':
             fs_root = os.path.join(tempfile.gettempdir(), 'workdir')
+            # logger.info(f'use {fs_root} as working directory.')
+
         remote_root = os.path.abspath(os.path.expanduser(fs_root))
-        if not fs.exists(remote_root):
-            fs.mkdirs(remote_root, exist_ok=True)
+        try:
+            if not fs.exists(remote_root):
+                fs.mkdirs(remote_root, exist_ok=True)
+            else:
+                with tempfile.TemporaryFile(prefix='hyn_', dir=remote_root) as t:
+                    t.write(b'.')
+        except PermissionError as e:
+            logger.warn(f'{type(e).__name__}: working directory "{remote_root}"')
+            logger.warn(e)
 
         local_root = remote_root
+        is_local = True
     else:
         remote_root = fs_root if fs_root else '/tmp'
         if not fs.exists(remote_root):
             fs.mkdirs(remote_root, exist_ok=True)
 
-        local_root = config('storage_localroot', os.path.join(tempfile.gettempdir(), 'cache'))
+        local_root = StorageCfg.local_root
         local_root = os.path.abspath(os.path.expanduser(local_root))
         os.makedirs(local_root, exist_ok=True)
+        is_local = False
 
     # return fix_filesystem(fs, remote_root, local_root)
     if type(fs).__name__ == 'S3FileSystem':
         return S3FileSystemAdapter(remote_root, local_root, fs.sep)(fs)
     else:
-        return FileSystemAdapter(remote_root, local_root, fs.sep)(fs)
+        return FileSystemAdapter(remote_root, local_root, os.path.sep if is_local else fs.sep)(fs)
 
 
 def _fs_reduce(*args):
-    return get_filesystem, (config('storage_type', 'file'),
-                            config('storage_root', None),
-                            config('storage_options', None))
+    return get_filesystem, (StorageCfg.kind if len(StorageCfg.kind) > 0 else None,
+                            StorageCfg.root if len(StorageCfg.root) > 0 else None,
+                            StorageCfg.options if len(StorageCfg.options) > 0 else None)
 
 
 filesystem = _fs_reduce()[0](*_fs_reduce()[1])
